class FastAPIProvider:
    def __init__(self, request):
        self.request = request

    @property
    def auth_token(self):
        return self.request.headers.get('Authorization')

    @property
    def correlation_id(self):
        return self.request.headers.get('X-Correlation-Id')

    @property
    def locale(self):
        return self.request.headers.get('Accept-Language')



from typing import Any

from elefantolib import constants


class BaseClient:

    def __init__(self, service_name: str, platform_context: dict[str, Any], api_url: str = None):
        self.service_name = service_name
        self.api_url = api_url or constants.SERVICE_MAIN_URL.format(service_name)
        self.platform_context = platform_context

    @property
    def headers(self) -> dict[str, Any]:
        return {
            'Accept-Language': self.platform_context.get('locale'),
            'X-Correlation-Id': self.platform_context.get('correlation_id'),
        }



from typing import Any, Protocol


class Provider(Protocol):

    def auth_token(self):
        ...

    def correlation_id(self):
        ...

    def locale(self):
        ...


class DictProvider:

    def __init__(self, context: dict[str, Any]):
        self.context = context

    @property
    def auth_token(self):
        return self.context.get('auth_token')

    @property
    def correlation_id(self):
        return self.context.get('correlation_id')

    @property
    def locale(self):
        return self.context.get('locale')



from typing import Any


class ContextService:

    def __init__(self, http_client, ws_client, context: dict[str, Any]):
        self._http_client = http_client
        self._ws_client = ws_client
        self._context = context
        self.__services = {}

    def __getattr__(self, item):
        if service := self.__services.get(item):
            return service

        self.__services[item] = type(
            item,
            (self._http_client, self._ws_client),
            {'service_name': item, 'platform_context': self._context},
        )

        return self.__services[item]

    def __getitem__(self, item):
        return getattr(self, item)



from typing import Any

from elefantolib import constants


class BaseWebsocketClient:

    def __init__(self, service_name: str, platform_context: dict[str, Any], api_url: str = None):
        self.service_name = service_name
        self.platform_context = platform_context

        ws_url = api_url or constants.SERVICE_MAIN_URL.format(service_name)
        self.ws_url = ws_url.replace('http', 'ws')



from elefantolib.constants import TOKEN_HEADER_TYPES

from httpx import codes

from jwt import PyJWTError


class BaseDetailedException(Exception):

    def __init__(self, detail: str, status_code: int):
        self.detail = detail
        self.status_code = status_code

    def __repr__(self) -> str:
        return f'{self.status_code}: {self.detail}'


class ClientError(BaseDetailedException):
    STATUS = codes.BAD_REQUEST

    def __init__(self, detail: str = 'invalid request', status_code: int = codes.BAD_REQUEST):
        super().__init__(detail, status_code)


class TokenError(PyJWTError, ClientError):
    STATUS = codes.UNAUTHORIZED


class EmptyTokenError(TokenError):
    STATUS = codes.UNAUTHORIZED

    def __str__(self):
        return 'Token is not provided'


class TokenLengthError(TokenError):
    STATUS = codes.UNAUTHORIZED

    def __init__(self, token):
        self.length = token.split()

    def __str__(self):
        return f'Token must contain 2 parts, got {self.length}'


class UnsupportedTokenType(TokenError):
    STATUS = codes.UNAUTHORIZED

    def __init__(self, token):
        self.type = token.split()[0]

    def __str__(self):
        return f'Supported token types is {TOKEN_HEADER_TYPES}, got {self.type}'



from typing import Iterable, NoReturn

from elefantolib import exceptions, provider
from elefantolib.auth import UserTokenService
from elefantolib.context import context_service
from elefantolib.http_client import httpx_client
from elefantolib.websocket_client import websocket_client


class BaseContext:
    HTTP_CLIENT = None
    WEBSOCKET_CLIENT = None
    user = None
    CONTEXT_ATTRIBUTES = (
        'auth_token',
        'correlation_id',
        'locale',
    )
    REQUIRED_ATTRIBUTES = (
        'auth_token',
        'correlation_id',
        'locale',
    )

    def __init__(self, pvr: provider.Provider):
        for attribute in self.CONTEXT_ATTRIBUTES:
            setattr(self, attribute, getattr(pvr, attribute, None))

        self.services = context_service.ContextService(self.HTTP_CLIENT, self.WEBSOCKET_CLIENT, self.context)

    @property
    def context(self):
        return {a: getattr(self, a) for a in self.CONTEXT_ATTRIBUTES}

    def validate(self, required_attrs: Iterable = None) -> None | NoReturn:
        required_attrs = required_attrs or self.REQUIRED_ATTRIBUTES

        for required_attr in required_attrs:
            if required_attr == 'auth_token' and (token := getattr(self, required_attr, None)):
                uts = UserTokenService()
                self.user = uts.extract_user(token)
                continue

            if getattr(self, required_attr, None) is not None:
                continue

            raise exceptions.ClientError(f'{required_attr} must be set for this request.')

        return None


class PlatformContext(BaseContext):
    HTTP_CLIENT = httpx_client.HttpxClient
    WEBSOCKET_CLIENT = websocket_client.WebsocketClient


class AsyncPlatformContext(BaseContext):
    HTTP_CLIENT = httpx_client.AsyncHttpxClient
    WEBSOCKET_CLIENT = websocket_client.WebsocketClient



class DjangoProvider:

    def __init__(self, request):
        self.request = request

    @property
    def auth_token(self):
        return self.request.META.get('HTTP_AUTHORIZATION')

    @property
    def correlation_id(self):
        return self.request.META.get('HTTP_X_CORRELATION_ID')

    @property
    def locale(self):
        return self.request.META.get('HTTP_ACCEPT_LANGUAGE')



import logging
from typing import NoReturn

from elefantolib.auth import user
from elefantolib.constants import ALGORITHM, ISSUER, SECRET, TOKEN_HEADER_TYPES
from elefantolib.exceptions import EmptyTokenError, TokenError, TokenLengthError, UnsupportedTokenType

import jwt


logger = logging.getLogger(__name__)


class UserTokenService:
    SECRET = SECRET
    ALGORITHM = ALGORITHM
    ISSUER = ISSUER

    def extract_token(self, token: str | None = None) -> dict | NoReturn:

        try:
            if not token:
                raise EmptyTokenError()

            parts = token.split()

            if len(parts) != 2:
                raise TokenLengthError(token)

            if parts[0] not in TOKEN_HEADER_TYPES:
                raise UnsupportedTokenType(token)

            return jwt.decode(
                jwt=parts[1],
                key=self.SECRET,
                algorithms=[self.ALGORITHM],
                issuer=ISSUER,
                options={
                    'verify_exp': True,
                    'verify_iss': True,
                },
            )

        except (jwt.ExpiredSignatureError,
                jwt.InvalidIssuerError,
                jwt.InvalidSignatureError,
                jwt.InvalidSignatureError) as e:
            logger.error(TokenError(f'Problems with token: {e}'))
            raise TokenError(f'Problems with token: {e}')

    def extract_user(self, token: str = None) -> user.BaseUser:
        payload = self.extract_token(token)

        if not {'id', 'name'}.issubset(set(payload.keys())):
            return user.AnonymousUser()

        return user.User(
            payload['id'],
            payload['name'],
            payload.get('email', None),
            payload.get('permissions', set()),
        )



import logging


def configure_logger(name: str) -> logging.Logger:
    logger = logging.getLogger(name)
    logger.setLevel(logging.INFO)
    formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    stream_handler = logging.StreamHandler()
    stream_handler.setFormatter(formatter)
    logger.addHandler(stream_handler)
    return logger



import os

SERVICE_MAIN_URL = 'http://{}:8000'
SECRET = os.environ.get('SECRET', None)
ALGORITHM = os.environ.get('ALGORITHM', 'HS256')
ISSUER = os.environ.get('ISSUER')

TOKEN_HEADER_TYPES = ('Bearer', 'JWT')



import uuid
from typing import NamedTuple, Protocol


class BaseUser(Protocol):
    id: uuid.UUID
    name: str
    email: str | None
    permissions: set[str]

    @staticmethod
    def is_authenticated() -> bool:
        ...

    def has_permission(self, permission: str) -> bool:
        ...

    def has_permissions(self, permissions: set[str]) -> bool:
        ...


class AnonymousUser:
    id: uuid.UUID
    name: str
    email: str | None
    permissions: set[str]

    @staticmethod
    def is_authenticated() -> bool:
        return False

    def has_permission(self, permission: str) -> bool:
        return False

    def has_permissions(self, permissions: set[str]) -> bool:
        return False


class User(NamedTuple):
    id: uuid.UUID = None
    name: str = None
    email: str | None = None
    permissions: set[str] = set()

    @staticmethod
    def is_authenticated() -> bool:
        return True

    def has_permission(self, permission: str) -> bool:
        return permission in self.permissions

    def has_permissions(self, permissions: set[str]) -> bool:
        return permissions.issubset(self.permissions)



import logging
from functools import partial

from elefantolib import logger_config
from elefantolib.http_client import BaseClient

import httpx


logger = logger_config.configure_logger(__name__)


class HttpxClient(BaseClient):

    def __getattr__(self, item):
        return partial(self._request, method=item)

    def _request(self,
                 path: str,
                 raises: bool = False,
                 success_log_level: str = 'INFO',
                 error_log_level: str = 'ERROR',
                 **kwargs):
        method = kwargs.pop('method', 'get')

        logger.setLevel(getattr(logging, success_log_level))
        log = getattr(logger, success_log_level.lower())

        with httpx.Client(headers=self.headers) as client:
            kwargs['headers'] = {**self.headers, **kwargs.get('headers', {})}

            log(f'\nREQUEST: {method.upper()}\nURL: {self.api_url}/{path}'
                f'\nHEADERS: {kwargs["headers"]}\nBODY: {kwargs.get("data")}')

            try:
                resp = getattr(client, method)(f'{self.api_url}/{path}', **kwargs)

                if resp.is_error:
                    logger.setLevel(getattr(logging, error_log_level))
                    log = getattr(logger, error_log_level.lower())

                log(f'\nRESPONSE: {resp.status_code}'
                    f'\nHEADERS: {resp.headers}\nBODY: {resp.content.decode("utf-8")}')

                return (resp, None) if not raises else resp
            except Exception as e:
                logger.setLevel(getattr(logging, error_log_level))
                log = getattr(logger, error_log_level.lower())
                log(e)
                return (None, e) if not raises else None


class AsyncHttpxClient(BaseClient):

    def __getattr__(self, item):
        return partial(self._request, method=item)

    async def _request(self, method: str, path: str, raises: bool = False, **kwargs):
        async with httpx.AsyncClient(headers=self.headers) as client:
            kwargs['headers'] = {**self.headers, **kwargs.get('headers', {})}

            try:
                resp = await getattr(client, method)(f'{self.api_url}/{path}', **kwargs)

                return (resp, None) if not raises else resp
            except BaseException as e:
                logger.error(e)
                return (None, e) if not raises else None



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['elefantolib',
 'elefantolib.auth',
 'elefantolib.context',
 'elefantolib.http_client',
 'elefantolib.provider',
 'elefantolib.provider.django_provider',
 'elefantolib.provider.fastapi_provider',
 'elefantolib.websocket_client']

package_data = \
{'': ['*']}

install_requires = \
['exceptiongroup>=1.1.1,<2.0.0',
 'httpx>=0.23.3,<0.24.0',
 'pyjwt>=2.6.0,<3.0.0',
 'pytest-env>=0.8.1,<0.9.0',
 'websockets>=11.0,<12.0']

setup_kwargs = {
    'name': 'elefantolib',
    'version': '0.9.2',
    'description': 'Elefanto lib',
    'long_description': 'Elefantolib\n-----------------------\n\n   **NOTE:** After clone this repository you should run command:\n\n   ``git config core.hooksPath .githooks``\n',
    'author': 'Elefanto',
    'author_email': 'elefanto@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/elefanto-organization/elefantolib',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



from elefantolib import context
from elefantolib.provider import django_provider


class DjangoPlatformContextMiddleware:

    def __init__(self, get_response):
        self._get_response = get_response

    def __call__(self, request, *args, **kwargs):
        pvr = django_provider.DjangoProvider(request=request)
        request.pfm = context.PlatformContext(pvr=pvr)
        request.pfm.validate()
        response = self._get_response(request)

        return response


class DjangoAsyncPlatformContextMiddleware:

    def __init__(self, get_response):
        self._get_response = get_response

    def __call__(self, request, *args, **kwargs):
        pvr = django_provider.DjangoProvider(request=request)
        request.async_pfm = context.AsyncPlatformContext(pvr=pvr)
        request.async_pfm.validate()
        response = self._get_response(request)

        return response



from typing import Callable

from elefantolib.websocket_client import BaseWebsocketClient

import websockets


class WebsocketClient(BaseWebsocketClient):

    async def ws(self, path: str, callback: Callable, *args, **kwargs):
        url = f'{self.ws_url}/{path}'
        async with websockets.connect(url) as ws_connect:
            await callback(ws_connect, *args, **kwargs)
